'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (method, url, data, useCache) {
  var withCredentials = true;

  if (arguments.length === 1) {
    var options = arguments[0];

    method = options.method || options.type || 'GET';
    useCache = options.useCache || options.cache;
    url = options.url;
    data = options.data;
    withCredentials = typeof options.withCredentials !== 'undefined' ? !!options.withCredentials : true;
  }

  var jsonData = JSON.stringify(data);
  var cacheValue = jsonData === undefined ? '' : jsonData;

  if (!useCache || !(cache[url] && cache[url][cacheValue])) {
    if (!cache[url]) cache[url] = [];

    // create a promise around an xhr object with json
    cache[url][cacheValue] = new Promise(function (resolve, reject) {
      var request = new XMLHttpRequest();

      request.open(method, url, true);

      // support cross origin requests
      request.setRequestHeader('Accept', '*/*');
      request.setRequestHeader('Content-type', 'application/json');
      request.withCredentials = withCredentials;

      request.onload = function () {
        if (request.status >= 200 && request.status < 300) {
          resolve(JSON.parse(request.response));
        } else {
          reject(request.statusText);
        }
      };

      request.onerror = function () {
        return reject('An error occurred in the request');
      };

      request.send(jsonData);
    });
  }

  return cache[url][cacheValue];
};

var cache = {};